unit clXmlUtils;

interface

uses
  Classes, msxml;

function GetArrayIndexByName(const AName: string; var Arr: array of string): Integer;
procedure StringsToXML(AStrings: TStrings; ANode: IXMLDOMNode);
procedure XMLToStrings(AStrings: TStrings; ANode: IXMLDOMNode);
procedure SaveXMLToFile(const AFileName: string; ADomDoc: IXMLDOMDocument);
procedure AddCDataNodeName(const AName: string);
function GetAttributeValue(const ANode: IXMLDOMNode; const AName: string): string;
procedure SetAttributeValue(const ANode: IXMLDOMNode; const AName, AValue: string);

implementation

uses
  SysUtils{$IFDEF VER140}, Variants{$ENDIF};

const
  CDataXMLFormat = '<xsl:output indent="yes" method="xml" encoding="UTF-8" cdata-section-elements="%s"/>';
  XMLFormat = '<?xml version="1.0" encoding="UTF-8"?>' +
              '<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">' +
              '  <xsl:output indent="yes" method="xml" encoding="UTF-8"/> %s ' +
              '    <xsl:template match="/ | @* | node()">' +
              '  <xsl:copy>' +
              '  <xsl:apply-templates select="@* | node()"/>' +
              '    </xsl:copy>' +
              '  </xsl:template>' +
              '</xsl:stylesheet>';

var
  CDataNodeNames: TStrings = nil;

procedure AddCDataNodeName(const AName: string);
begin
  if (CDataNodeNames = nil) then
  begin
    CDataNodeNames := TStringList.Create();
  end;
  if (CDataNodeNames.IndexOf(AName) < 0) then
  begin
    CDataNodeNames.Add(AName);
  end;
end;
              
procedure SaveXMLToFile(const AFileName: string; ADomDoc: IXMLDOMDocument);
  function GetXMLFormat: string;
  var
    i: Integer;
  begin
    Result := '';
    if (CDataNodeNames <> nil) then
    begin
      for i := 0 to CDataNodeNames.Count - 1 do
      begin
        Result := Result + #32 + Format(CDataXMLFormat, [CDataNodeNames[i]]);
      end;
    end;
    Result := Format(XMLFormat, [Result]);
  end;
  
var
  b: Boolean;
  TransDoc, ResDoc: IXMLDOMDocument;
begin
  TransDoc := CoDOMDocument.Create();
  ResDoc := CoDOMDocument.Create();
  TransDoc.loadXML(GetXMLFormat());
  try
    ADomDoc.transformNodeToObject(TransDoc, ResDoc);
    b := (ResDoc.xml <> '');
  except
    b := False;
  end;
  if b then
  begin
    ResDoc.save(AFileName);
  end else
  begin
    ADomDoc.save(AFileName);
  end;
end;

function GetArrayIndexByName(const AName: string; var Arr: array of string): Integer;
begin
  for Result := Low(Arr) to High(Arr) do
  begin
    if (CompareText(Arr[Result], AName) = 0) then Exit;
  end;
  Result := -1;
end;

procedure StringsToXML(AStrings: TStrings; ANode: IXMLDOMNode);
var
  ChildNode: IXMLDOMNode;
begin
  ChildNode := ANode.ownerDocument.createCDATASection(AStrings.Text);
  ANode.appendChild(ChildNode);
end;

procedure XMLToStrings(AStrings: TStrings; ANode: IXMLDOMNode);
begin
  AStrings.Text := ANode.text;
end;

function GetAttributeValue(const ANode: IXMLDOMNode; const AName: string): string;
begin
  Result := VarToStr((ANode as IXMLDOMElement).getAttribute(AName));
end;

procedure SetAttributeValue(const ANode: IXMLDOMNode; const AName, AValue: string);
begin
  if (AValue <> '') then
  begin
    (ANode as IXMLDOMElement).setAttribute(AName, AValue);
  end;
end;

initialization

finalization
  CDataNodeNames.Free();

end.